#include "Driver.h"
#include "nvsk_sys.h"
#include "nv_emul.h"
#include "utils.h"

VOID     UnloadRoutine(IN PDRIVER_OBJECT pDriverObject);
NTSTATUS ReadHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp);
NTSTATUS CreateHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp);
NTSTATUS CloseHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp);

#pragma code_seg("INIT")

extern "C"
NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath)
{
	NTSTATUS status = STATUS_SUCCESS;
	PDEVICE_OBJECT fdo;	
	UNICODE_STRING devName;
	OBJECT_ATTRIBUTES oa;
	
	DriverObject->DriverUnload = UnloadRoutine;
	DriverObject->MajorFunction[IRP_MJ_CREATE] = CreateHandler;
	DriverObject->MajorFunction[IRP_MJ_CLOSE] = CloseHandler;
	DriverObject->MajorFunction[IRP_MJ_READ] = ReadHandler;

	RtlInitUnicodeString(&devName, L"\\Device\\DF");

	status = IoCreateDevice(DriverObject, sizeof(FILTER_DEVICE_EXTENSION),
		&devName, FILE_DEVICE_UNKNOWN, 0, FALSE, &fdo);

	if (!NT_SUCCESS(status)) return status;

	PFILTER_DEVICE_EXTENSION dx = (PFILTER_DEVICE_EXTENSION)fdo->DeviceExtension;
	dx->fdo = fdo;

	#if DBG
	dx->m_DemoCounter = 0;
	KeQueryTickCount((LARGE_INTEGER *)&dx->t);
	#endif

	RtlCopyBytes(dx->ustrMainKey, RegistryPath->Buffer, RegistryPath->Length);

	#if DBG
	DbgPrint("==DF> DriverEntry. Reg: %ws\n", dx->ustrMainKey);
	#endif

	UNICODE_STRING GuardantName;
	#define DONGLE_GUARDANT_NAME L"\\DosDevices\\NVKEYNT"
	RtlInitUnicodeString(&GuardantName, DONGLE_GUARDANT_NAME);

	PFILE_OBJECT pFileObj;
	PDEVICE_OBJECT pDevObj;

	dx->pGuardantDevObj = NULL;
	status = IoGetDeviceObjectPointer(&GuardantName, FILE_ALL_ACCESS, &pFileObj, &pDevObj);
	if (!NT_SUCCESS(status)) 
	{
		#if DBG
		DbgPrint("==DF> DriverEntry. IoGetDeviceObjectPointer error: 0x%X\n", status);
		#endif
	}
	else
	{
	        dx->pGuardantDevObj = IoAttachDeviceToDeviceStack(fdo, pDevObj);
		if(dx->pGuardantDevObj)
		{
        		fdo->Flags |= (dx->pGuardantDevObj->Flags & DO_BUFFERED_IO);
			#if DBG
			DbgPrint("==DF> Attached to Guardant driver! 0x%X\n", dx->pGuardantDevObj);
			#endif
		}
	}
	return STATUS_SUCCESS;
}

#pragma code_seg()

#pragma code_seg("PAGE")

NTSTATUS CompleteIrp(PIRP Irp, NTSTATUS status, ULONG info)
{
	Irp->IoStatus.Status = status;
	Irp->IoStatus.Information = info;
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	return status;
}

NTSTATUS ReadHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp)
{
	ULONG BytesTxd = 0, c = 0;
	NTSTATUS status = STATUS_SUCCESS;
	PIO_STACK_LOCATION IrpSp;
	ns_Args *ns_buf;

	#if DBG
		PFILTER_DEVICE_EXTENSION tmp_dx = (PFILTER_DEVICE_EXTENSION)fdo->DeviceExtension;
		__int64 t;
		KeQueryTickCount((LARGE_INTEGER *)&t);
		DbgPrint("==DF> ReadHandler\n>> Time = %d\n", t - tmp_dx->t);
		if (t - tmp_dx->t >= 50000) // >= 15000
			return CompleteIrp(Irp, status, BytesTxd);
	#endif
	

	IrpSp = IoGetCurrentIrpStackLocation(Irp);

	#if DBG
		DbgPrint("==DF> ReadHandler\n>> Length = %d\n", IrpSp->Parameters.Read.Length);
	#endif

	if (Irp->UserBuffer)
	{
		if (IrpSp->Parameters.Read.Length == 438) // Guardant Stealth Packet Size
		{
			ns_buf = (ns_Args *)Irp->UserBuffer;
			FullDecode((BYTE *)ns_buf, IrpSp->Parameters.Read.Length);

			#if DBG
			DbgPrint("==DF> ReadHandler\n>> Ver = 0x%X; Func = 0x%X; Addr = 0x%X; Len = 0x%X; PrivateCode = 0x%X; PrCodeStr = %s\n", ns_buf->naVer, ns_buf->na_bCmd, ns_buf->na_bAddr, ns_buf->na_bLen, ns_buf->na_dwPrivate, ns_buf->na_szSprivate);
			#endif

			HANDLE hMainKey;
			OBJECT_ATTRIBUTES oa;
			PFILTER_DEVICE_EXTENSION dx = (PFILTER_DEVICE_EXTENSION)fdo->DeviceExtension;
			UNICODE_STRING utmp;
			RtlInitUnicodeString(&utmp, dx->ustrMainKey);
			InitializeObjectAttributes(&oa, &utmp, OBJ_CASE_INSENSITIVE, NULL, NULL);
			status = ZwOpenKey(&hMainKey, KEY_QUERY_VALUE, &oa);
			
			#if DBG			
			DbgPrint("==DF> DriverEntry. Reg: %ws. Error: 0x%.8X\n", utmp.Buffer, status);
			#endif

			if (NT_SUCCESS(status))
			{
				#if DBG			
					DbgPrint("==DF> DriverEntry. RegKey successufully opened.\n");
					GuardantEmulator(ns_buf, IrpSp->Parameters.Read.Length, hMainKey, &dx->m_DemoCounter);
					// DbgPrint("==DF> ReadOutput.\n");
				#else
					GuardantEmulator(ns_buf, IrpSp->Parameters.Read.Length, hMainKey, NULL);
				#endif

				ZwClose(&hMainKey);
			}

			FullDecode((BYTE *)ns_buf, IrpSp->Parameters.Read.Length);
		}
	}

	return CompleteIrp(Irp, status, BytesTxd);
}

NTSTATUS CreateHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp)
{
	#if DBG
	DbgPrint("==DF> CreateHandler.\n");
	#endif
	return CompleteIrp(Irp, STATUS_SUCCESS, 0);
}

NTSTATUS CloseHandler(IN PDEVICE_OBJECT fdo, IN PIRP Irp)
{
	#if DBG
	DbgPrint("==DF> CloseHandler.\n");
	#endif
	return CompleteIrp(Irp, STATUS_SUCCESS, 0);
}

VOID UnloadRoutine(IN PDRIVER_OBJECT pDriverObject)
{
	PDEVICE_OBJECT pNextDevObj;
	int i = 0;

	#if DBG
		DbgPrint("==DF> UnloadRoutine\n");
	#endif

	pNextDevObj = pDriverObject->DeviceObject;
	for(; pNextDevObj != NULL; ++i)
	{
		PFILTER_DEVICE_EXTENSION dx = (PFILTER_DEVICE_EXTENSION)pNextDevObj->DeviceExtension;
		pNextDevObj = pNextDevObj->NextDevice;

		if (dx->pGuardantDevObj)
		        IoDetachDevice(dx->pGuardantDevObj);

		if (dx->fdo)
			IoDeleteDevice(dx->fdo);
	}
}

#pragma code_seg()